-----------------------------------------------------------------------
--
--   Beispielskript zum Vortrag "Access und SQL Server-Verschlsselung"
--
--   Autor: Bernd Jungbluth										
--   Keine Gewhrleistung, keine Haftung
--
-----------------------------------------------------------------------


USE master;
GO

-- ###############################
-- Datenbank
-- ###############################

RAISERROR('Anlegen der Datenbank', 10, 1);
GO

CREATE DATABASE semKrypto; 
GO


USE semKrypto;
GO

RAISERROR('Anlegen der Tabellen', 10, 1);
GO

-- ###############################
-- Tabellen
-- ###############################


CREATE TABLE dbo.Stellen
(	
	StellenID			INT				NOT NULL	IDENTITY(1,1) ,
	Stelle				NVARCHAR(50)	NULL,
	Stellenbeschreibung	NVARCHAR(MAX)	NOT NULL,
	ZuBesetzenAb		DATE			NOT NULL,
	iTS					TIMESTAMP	NOT NULL,
	CONSTRAINT PK__Stellen_StellenID PRIMARY KEY CLUSTERED (StellenID ASC),
	CONSTRAINT UK__Stellen_Stelle_ZuBesetzenAb UNIQUE (Stelle ASC, ZuBesetzenAb ASC)
);
GO

CREATE TABLE dbo.Bewerber
(	
	BewerberID		INT				NOT NULL	IDENTITY(1,1) ,
	StellenID		INT				NULL,
	Nachname		NVARCHAR(20)	NOT NULL,
	Vorname			NVARCHAR(10)	NOT NULL,
	Religion		NVARCHAR(50)	NULL,
	Strasse			NVARCHAR(25)	NOT NULL,
	Hausnummer		NVARCHAR(25)	NOT NULL,
	PLZ				NVARCHAR(10)	NOT NULL,
	Ort				NVARCHAR(50)	NOT NULL,
	Geburtsdatum	DATETIME2(0)	NULL,
	Mobil			NVARCHAR(4)		NULL,
	Email			NVARCHAR(255)	NULL,
	Gehaltswunsch	MONEY			NOT NULL,
	Bemerkungen		NVARCHAR(4000)	NULL,
	iTS				TIMESTAMP		NOT NULL,
	CONSTRAINT PK__Bewerber_BewerberID PRIMARY KEY CLUSTERED (BewerberID ASC),
	CONSTRAINT UK__Bewerber_StellenID UNIQUE (StellenID ASC, Email ASC)
);
GO

CREATE TABLE dbo.Mitarbeiter
(	
	MitarbeiterID		INT				NOT NULL	IDENTITY(1,1),
	PersonalNr			CHAR(12)		NOT NULL,
	Nachname			NVARCHAR(20)	NOT NULL,
	Vorname				NVARCHAR(10)	NOT NULL,
	Religion			NVARCHAR(50)	NULL,
	crptReligion		VARBINARY(MAX)	NULL,
	Bewertung			CHAR(1)			NULL,
	crptBewertung		VARBINARY(MAX)	NULL,
	Position			NVARCHAR(30)	NULL,
	Anrede				NVARCHAR(25)	NOT NULL,
	Geburtsdatum		DATETIME2(0)	NULL,
	Einstellung			DATETIME2(0)	NOT NULL,
	Durchwahl			NVARCHAR(4)		NULL,
	Mobil				NVARCHAR(4)		NULL,
	crptMobil			VARBINARY(MAX)	NULL,
	Email				NVARCHAR(255)	NULL,
	Bemerkungen			NVARCHAR(4000)	NULL,
	Vorgesetzter		INT				NULL,
	AktuellesGehalt		MONEY			NOT NULL,
	iTS					TIMESTAMP		NOT NULL,
	CONSTRAINT PK__Mitarbeiter_MitarbeiterID PRIMARY KEY CLUSTERED (MitarbeiterID ASC),
	CONSTRAINT UK__Mitarbeiter_PersonalNr UNIQUE (PersonalNr ASC)
);
GO

-- Fremdschlssel Stellen
ALTER TABLE dbo.Bewerber WITH CHECK 
	ADD CONSTRAINT FK__Bewerber_Stellen_StellenID FOREIGN KEY(StellenID) REFERENCES dbo.Stellen (StellenID);
ALTER TABLE dbo.Bewerber CHECK CONSTRAINT FK__Bewerber_Stellen_StellenID;


-- ###############################
-- Daten
-- ###############################

RAISERROR('Anfgen der Daten', 10, 1);
GO

SET NOCOUNT ON;
SET IDENTITY_INSERT dbo.Mitarbeiter ON 
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (1, N'HP1988020800', N'Hesselbach', N'Petra', N'Vulkanisch', NULL, 'A', NULL, N'Geschftsfhrerin', N'Frau', CAST(N'1988-02-08T00:00:00.0000000' AS DateTime2), CAST(N'2011-06-01T00:00:00.0000000' AS DateTime2), N'200', NULL, N'petra.hesselbach@coolesocke.com', NULL, NULL, 3000.0000)
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (2, N'HA1992081900', N'Hoppenstett', N'Andreas', N'Keltisch', NULL, 'A', NULL, N'Vertriebsleiter', N'Herr', CAST(N'1992-08-19T00:00:00.0000000' AS DateTime2), CAST(N'2000-02-01T00:00:00.0000000' AS DateTime2), N'100', NULL, N'andreas.hoppenstett@coolesocke.com', NULL, 1, 7500.0000)
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (3, N'BA1993093000', N'Bienlein', N'Anna', N'Keltisch', NULL, 'A', NULL, N'Vertrieb', N'Frau', CAST(N'1993-09-30T00:00:00.0000000' AS DateTime2), CAST(N'2010-12-01T00:00:00.0000000' AS DateTime2), N'146', NULL, N'anna.bienlein@coolesocke.com', NULL, 2, 2500.0000)
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (4, N'BR1998011900', N'Brner', N'Roland', N'Keltisch', NULL, 'B', NULL, N'Vertrieb', N'Herr', CAST(N'1998-01-19T00:00:00.0000000' AS DateTime2), CAST(N'2007-01-01T00:00:00.0000000' AS DateTime2), N'147', NULL, N'roland.boerner@coolesocke.com', NULL, 2, 3333.0000)
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (5, N'BS1987040300', N'Beimer', N'Susanne', N'Keltisch', NULL, 'B', NULL, N'Vertrieb', N'Frau', CAST(N'1987-04-03T00:00:00.0000000' AS DateTime2), CAST(N'2001-05-01T00:00:00.0000000' AS DateTime2), N'141', NULL, N'susanne.beimer@coolesocke.com', NULL, 2, 5000.0000)
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (6, N'LM1986020700', N'Ldenscheid', N'Melanie', N'Keltisch', NULL, 'C', NULL, N'Einkaufsleiter', N'Frau', CAST(N'1986-02-07T00:00:00.0000000' AS DateTime2), CAST(N'2003-04-01T00:00:00.0000000' AS DateTime2), N'142', NULL, N'melanie.luedenscheid@coolesocke.com', NULL, 1, 3100.0000)
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (7, N'BH1996092900', N'Becker', N'Holger', N'Keltisch', NULL, 'B', NULL, N'Einkauf', N'Herr', CAST(N'1996-09-29T00:00:00.0000000' AS DateTime2), CAST(N'2001-01-02T00:00:00.0000000' AS DateTime2), N'143', NULL, N'holger.becker@coolesocke.com', NULL, 6, 3300.0000)
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (8, N'BM1998090100', N'Blmel', N'Marina', N'Keltisch', NULL, 'A', NULL, N'Einkauf', N'Frau', CAST(N'1998-09-01T00:00:00.0000000' AS DateTime2), CAST(N'2005-09-01T00:00:00.0000000' AS DateTime2), N'144', NULL, N'marina.bluemel@coolesocke.com', NULL, 6, 3300.0000)
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (9, N'SE1995042200', N'Schulz', N'Ellen', N'Keltisch', NULL, 'B', NULL, N'Einkauf', N'Frau', CAST(N'1995-04-22T00:00:00.0000000' AS DateTime2), CAST(N'2002-11-01T00:00:00.0000000' AS DateTime2), N'145', NULL, N'ellen.schulz@coolesocke.com', NULL, 6, 3000.0000)
INSERT dbo.Mitarbeiter (MitarbeiterID, PersonalNr, Nachname, Vorname, Religion, crptReligion, Bewertung, crptBewertung, Position, Anrede, Geburtsdatum, Einstellung, Durchwahl, Mobil, Email, Bemerkungen, Vorgesetzter, AktuellesGehalt) VALUES (10, N'SK1989120100', N'Stromberg', N'Katrin', N'Keltisch', NULL, 'C', NULL, N'Personal', N'Frau', CAST(N'1989-12-01T00:00:00.0000000' AS DateTime2), CAST(N'2005-10-01T00:00:00.0000000' AS DateTime2), N'149', NULL, N'katrin.stromberg@coolesocke.com', NULL, 1, 3200.0000)
SET IDENTITY_INSERT dbo.Mitarbeiter OFF
GO

SET IDENTITY_INSERT dbo.Stellen ON 
INSERT dbo.Stellen (StellenID, Stelle, Stellenbeschreibung, ZuBesetzenAb) VALUES (1, N'Datenschutzbeauftragter', N'Qualitfiziert nach DSGVO und BDSG', N'20180525')
INSERT dbo.Stellen (StellenID, Stelle, Stellenbeschreibung, ZuBesetzenAb) VALUES (2, N'Sicherheitsbeauftragter', N'White-Hacker', N'20190525')
SET IDENTITY_INSERT dbo.Stellen OFF
GO

SET IDENTITY_INSERT dbo.Bewerber ON 
INSERT dbo.Bewerber (BewerberID, StellenID, Nachname, Vorname, Religion, Strasse, Hausnummer, PLZ, Ort, Geburtsdatum, Mobil, Email, Gehaltswunsch, Bemerkungen) VALUES (1, N'1', N'Mller-Ldenscheid', N'Karl-Heinz', N'keltisch', N'Bulwstr.', N'5', N'60528', N'Frankfurt', CAST(N'2000-11-11T00:00:00.0000000' AS DateTime2), NULL, N'kh@mueller-luedenscheid.de', 4000.0000, NULL)
INSERT dbo.Bewerber (BewerberID, StellenID, Nachname, Vorname, Religion, Strasse, Hausnummer, PLZ, Ort, Geburtsdatum, Mobil, Email, Gehaltswunsch, Bemerkungen) VALUES (2, N'1', N'Meier', N'Brigitte', N'keltisch', N'Kantenstr.', N'77', N'55469', N'Keidelheim', CAST(N'2000-01-10T00:00:00.0000000' AS DateTime2), NULL, N'maria@hoppenstett.de', 4000.0000, NULL)
SET IDENTITY_INSERT dbo.Bewerber OFF
GO
SET NOCOUNT OFF;


-- ###############################
-- Gespeicherte Prozeduren 
-- ###############################

RAISERROR('Gespeicherte Prozeduren anlegen', 10, 1);
GO

CREATE PROC dbo.pInsertBewerber	@Nachname	NVARCHAR(20),
								@Vorname	NVARCHAR(10),
								@Strasse	NVARCHAR(25),
								@Hausnr		NVARCHAR(25),
								@PLZ		NVARCHAR(10),
								@Ort		NVARCHAR(50),
								@Gehalt		MONEY,
								@Religion	NVARCHAR(50)
AS
SET NOCOUNT ON;

DECLARE @varNeueID As INT;

INSERT INTO dbo.Bewerber (Nachname, Vorname, Strasse, Hausnummer, PLZ, Ort, Gehaltswunsch, Religion)
VALUES (@Nachname, @Vorname, @Strasse, @Hausnr, @PLZ, @Ort, @Gehalt, @Religion);

SET @varNeueID = (SELECT SCOPE_IDENTITY());

SELECT	Nachname, Vorname, Strasse, Hausnummer, PLZ, Ort, Gehaltswunsch, Religion
FROM	dbo.Bewerber
WHERE	BewerberID = @varNeueID;
GO


CREATE PROC dbo.pUpdateBewerberGehaltswunsch	@BewerberID	INT,
												@Gehaltswunsch	MONEY
AS

SET NOCOUNT ON;

UPDATE	dbo.Bewerber 
SET		Gehaltswunsch = @Gehaltswunsch
WHERE	BewerberID = @BewerberID;

SELECT	Nachname, Vorname, Strasse, Hausnummer, PLZ, Ort, Gehaltswunsch, Religion
FROM	dbo.Bewerber
WHERE	BewerberID = @BewerberID;
GO


CREATE PROC dbo.pSelectMitarbeiterReligion
AS
SET NOCOUNT ON;

	-- Daten zeigen
	SELECT	MitarbeiterID, Nachname, Religion, 
			CAST(DECRYPTBYKEY(crptReligion) AS NVARCHAR(50)) As crptReligion
	FROM	dbo.Mitarbeiter;

	-- Symmetrischen Schlssel explizit schlieen
	CLOSE SYMMETRIC KEY skeyMitarbeiterReligion;
GO



-- ###############################
-- Anmeldung und Benutzer
-- ###############################

RAISERROR('Anmeldung und Benutzer anlegen', 10, 1);
GO

USE master
GO
BEGIN TRY
	-- Anmeldung Hesselbach anlegen -> Windows-Benutzer muss existieren
	CREATE LOGIN [BJSEMINARE\Hesselbach] FROM WINDOWS WITH DEFAULT_DATABASE = semKrypto;
END TRY
BEGIN CATCH
	THROW
END CATCH
GO

USE semKrypto;
GO
BEGIN TRY
	-- Benutzer Hesselbach anlegen -> Anmeldung muss existieren
	CREATE USER Hesselbach FOR LOGIN [BJSEMINARE\Hesselbach];
	ALTER ROLE db_datareader ADD MEMBER Hesselbach;
	ALTER ROLE db_datawriter ADD MEMBER Hesselbach;
	GRANT EXECUTE ON dbo.pSelectMitarbeiterReligion TO Hesselbach;
END TRY
BEGIN CATCH
	THROW
END CATCH
GO


-- ###############################
-- Ende
-- ###############################

RAISERROR('Fertig', 10, 1);
GO