-----------------------------------------------------------------------
--
--   Beispielskript zum Vortrag "Access und SQL Server-Verschlsselung"
--
--   Autor: Bernd Jungbluth										
--   Keine Gewhrleistung, keine Haftung
--
-----------------------------------------------------------------------


USE semKrypto;
GO

----------------------------------------------------------------
-- Zertifikat anlegen 
----------------------------------------------------------------

CREATE CERTIFICATE certMitarbeiterReligion
	WITH SUBJECT = 'Testzertifikat',
	EXPIRY_DATE = '20301231';
GO
--> Verschlsselt durch den per SMK geschtzten DMK


----------------------------------------------------------------
-- Symmetrischen Schlssel anlegen
----------------------------------------------------------------

CREATE SYMMETRIC KEY skeyMitarbeiterReligion
	WITH ALGORITHM = AES_256
	ENCRYPTION BY CERTIFICATE certMitarbeiterReligion;


----------------------------------------------------------------
-- Daten anzeigen
----------------------------------------------------------------

SELECT MitarbeiterID, Nachname, Religion, crptReligion FROM dbo.Mitarbeiter;


----------------------------------------------------------------
-- Per Symetrischem Schlssel verschlsseln
----------------------------------------------------------------

-- Wert der Spalte Religion verschlsselt in Spalte crptRelegion speichern
OPEN SYMMETRIC KEY skeyMitarbeiterReligion 
	DECRYPTION BY CERTIFICATE certMitarbeiterReligion;

UPDATE	dbo.Mitarbeiter
SET		crptReligion = ENCRYPTBYKEY(KEY_GUID('skeyMitarbeiterReligion'), Religion);

-- Daten lesen
SELECT	MitarbeiterID, Nachname, Religion, crptReligion As Original,
		CAST(DECRYPTBYKEY(crptReligion) AS NVARCHAR(50)) As crptReligion
FROM	dbo.Mitarbeiter;

CLOSE SYMMETRIC KEY skeyMitarbeiterReligion;
GO


----------------------------------------------------------------
-- Rechtevergabe
----------------------------------------------------------------

-- GRANT VIEW DEFINITION ON SYMMETRIC KEY::skeyMitarbeiterDMK TO Hesselbach;
-- GRANT CONTROL ON CERTIFICATE::certMitarbeiterDMK TO Hesselbach;

--> Wieder zu hohe Rechtevergabe auf Zertifikat
----> Also wie gehabt ber eine Gespeicherte Prozedur ...


----------------------------------------------------------------
-- Vorgehensweise
----------------------------------------------------------------

-- Rechtevergabe fr Hausmeister
GRANT VIEW DEFINITION ON SYMMETRIC KEY::skeyMitarbeiterReligion TO Hausmeister;
GRANT CONTROL ON CERTIFICATE::certMitarbeiterReligion TO Hausmeister;
GO


-- Absichern mit Gespeicherter Prozeduren
CREATE PROC dbo.pOeffneSchluesselReligion 
WITH
	EXECUTE AS 'Hausmeister'
AS
	OPEN SYMMETRIC KEY skeyMitarbeiterReligion 
		DECRYPTION BY CERTIFICATE certMitarbeiterReligion;
GO

-- Rechtevergabe an Gespeicherter Prozedur
GRANT EXECUTE ON dbo.pOeffneSchluesselReligion TO Hesselbach;
GO


-- Gespeicherte Prozedur erweitern
ALTER PROC dbo.pSelectMitarbeiterReligion
AS
SET NOCOUNT ON;

	EXEC dbo.pOeffneSchluesselReligion;

	-- Daten zeigen
	SELECT	MitarbeiterID, Nachname, Religion, 
			CAST(DECRYPTBYKEY(crptReligion) AS NVARCHAR(50)) As crptReligion
	FROM	dbo.Mitarbeiter;

	-- Symmetrischen Schlssel explizit schlieen
	CLOSE SYMMETRIC KEY skeyMitarbeiterReligion;
GO

