-----------------------------------------------------------------------
--
--   Beispielskript zum Vortrag "Access und SQL Server-Verschlsselung"
--
--   Autor: Bernd Jungbluth										
--   Keine Gewhrleistung, keine Haftung
--
-----------------------------------------------------------------------

USE semKrypto;
GO


----------------------------------------------------------------
-- Datenermittlung mit Always Encrypted
----------------------------------------------------------------

-- Aktueller Benutzer
SELECT SUSER_SNAME() As AktuellerBenutzer;



----------------------------------------------------------------
-- Notwendige Parametrisierung
----------------------------------------------------------------

SELECT Nachname, Vorname, PLZ, Religion FROM dbo.Bewerber WHERE PLZ = '60528';
GO 
--> Fehler



DECLARE @PLZ NVARCHAR(10) = N'60528';
SELECT Nachname, Vorname, PLZ, Religion FROM Bewerber WHERE PLZ = @PLZ;
GO 
--> Daten
--> blaue Intellisense-Markierung = erfolgreich parametrisiert



-- Typsicherheit beachten
DECLARE @PLZ VARCHAR(10) = '60528';
SELECT Nachname, Vorname, PLZ, Religion FROM Bewerber WHERE PLZ = @PLZ;
GO
--> Fehler



-- Keine separate Deklaration und Initializierung
DECLARE @PLZ NVARCHAR(10);
SET @PLZ = N'60528';
SELECT Nachname, Vorname, PLZ, Religion FROM Bewerber WHERE PLZ = @PLZ;
GO
--> Fehler



-- Keine Berechnungen
DECLARE @PLZ NVARCHAR(10) = N'605' + N'28';
SELECT Nachname, Vorname, PLZ, Religion FROM Bewerber WHERE PLZ = @PLZ;
GO
--> Fehler



-- Keine Systemvariablen
DECLARE @Geburtsdatum DATETIME2 = GETDATE();
SELECT Nachname, Vorname, PLZ, Geburtsdatum FROM Bewerber WHERE Geburtsdatum = @Geburtsdatum;
GO
--> Fehler



----------------------------------------------------------------
-- Daten filtern bei deterministischer Verschlsselung 
----------------------------------------------------------------

-- Daten filtern mit Ungleich
DECLARE @PLZ NVARCHAR(10) = '55469';
SELECT Nachname, Vorname, PLZ, Religion FROM Bewerber WHERE PLZ != @PLZ;
GO
--> Daten



-- Daten filtern mit Grer
DECLARE @PLZ NVARCHAR(10) = '55469';
SELECT Nachname, Vorname, PLZ, Religion FROM Bewerber WHERE PLZ > @PLZ;
GO
--> Fehler



-- Daten filtern mit LIKE
DECLARE @PLZ NVARCHAR(10) = '5%';
SELECT Nachname, Vorname, PLZ, Religion FROM Bewerber WHERE PLZ LIKE @Plz;
GO
--> Fehler



----------------------------------------------------------------
-- Daten filtern bei zuflliger Verschlsselung 
----------------------------------------------------------------

-- Daten filtern 
DECLARE @Gehaltswunsch MONEY = 4000
SELECT Nachname, Vorname, PLZ, Religion, Geburtsdatum, Gehaltswunsch FROM dbo.Bewerber 
WHERE Gehaltswunsch = @Gehaltswunsch;
GO
--> Fehler


